// FileLog.cpp: implementation of the CFileLog class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileLog.h"
//#include <time.h>
#include <shellapi.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileLog::CFileLog() : m_hFile(INVALID_HANDLE_VALUE)
{
	m_szLogFile[0] = (_TCHAR)NULL;
}

inline CFileLog::CFileLog(LPCTSTR lpszLogFile) : m_hFile(INVALID_HANDLE_VALUE)
{
	ResetFileName(lpszLogFile);
}

CFileLog::~CFileLog()
{
	Close();
}

BOOL CFileLog::Open()
{
	if (::_tcslen(m_szLogFile) == 0) return FALSE;
	if (m_hFile != INVALID_HANDLE_VALUE) return TRUE;
	if (::_tcsrchr(m_szLogFile, '\\') == NULL)
	{
		_TCHAR szPath[MAX_PATH]; 
		if (::GetModuleFileName(NULL, szPath, MAX_PATH) != 0)
		{
			_TCHAR* lpchLastBS = ::_tcsrchr(szPath, '\\');
			if (lpchLastBS != NULL)
			{
				szPath[(int)(lpchLastBS-szPath) + 1] = (_TCHAR)NULL;
				::_tcscat_s(szPath, m_szLogFile);
				::_tcscpy_s(m_szLogFile, szPath);
			}
		}
	}
	m_hFile = ::CreateFile(m_szLogFile,
		GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
		OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);                  
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		_TCHAR szBufDTNow[DATETIME_STR_LEN];
		struct tm timenow;
		::time_t long_time;
		::time(&long_time);
		errno_t err = ::localtime_s(&timenow, &long_time );
		_ASSERTE(err == 0);
		::_tcsftime(szBufDTNow, DATETIME_STR_LEN, DATETIME_FORMAT, &timenow);
		WriteStr(szBufDTNow);
	}
	return (m_hFile != INVALID_HANDLE_VALUE);
}

inline void CFileLog::Close()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		Flush();
		::CloseHandle(m_hFile);
		m_hFile = INVALID_HANDLE_VALUE;
	}
}

BOOL CFileLog::ResetFileName(LPCTSTR lpszLogFile)
{
	if (lpszLogFile != NULL)
	{
		Close();
		::_tcscpy_s(m_szLogFile, lpszLogFile);
		return Open();
	}
	return FALSE;
}

BOOL CFileLog::WriteLog(LPCTSTR lpszStr)
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		USES_CONVERSION;
		_TCHAR szBufTNow[TIME_STR_LEN];
		struct tm timenow;
		time_t long_time;
		::time(&long_time);
		errno_t err = ::localtime_s(&timenow, &long_time);
		_ASSERTE(err == 0);
		::_tcsftime(szBufTNow, TIME_STR_LEN, TIME_FORMAT, &timenow);
		DWORD nNumberOfBytesWritten = 0;
		if (::SetFilePointer(m_hFile, NULL, NULL, FILE_END) != INVALID_SET_FILE_POINTER)
		{
			if(WriteFile(m_hFile, (LPCVOID)"- [", 3, &nNumberOfBytesWritten, NULL) != 0 &&
				WriteFile(m_hFile, (LPCVOID)T2CA(szBufTNow), (DWORD)_tcslen(szBufTNow), &nNumberOfBytesWritten, NULL) != 0 &&
				WriteFile(m_hFile, (LPCVOID)"] ", 2, &nNumberOfBytesWritten, NULL) != 0 &&
				WriteFile(m_hFile, (LPCVOID)T2CA(lpszStr), (DWORD)_tcslen(lpszStr), &nNumberOfBytesWritten, NULL) != 0 &&
				WriteFile(m_hFile, (LPCVOID)STR_ENDLINE, LEN_ENDLINE, &nNumberOfBytesWritten, NULL) != 0 )
			{
				return TRUE;
			}
			else
				return FALSE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

inline BOOL CFileLog::WriteStr(LPCTSTR lpszStr)
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		USES_CONVERSION;
		DWORD nNumberOfBytesWritten = 0;
		if (::SetFilePointer(m_hFile, NULL, NULL, FILE_END) != INVALID_SET_FILE_POINTER)
		{
			if(WriteFile(m_hFile, (LPCVOID)T2CA(lpszStr), (DWORD)_tcslen(lpszStr), &nNumberOfBytesWritten, NULL) != 0 &&
				WriteFile(m_hFile, (LPCVOID)STR_ENDLINE, 2, &nNumberOfBytesWritten, NULL) != 0)
			{
				return TRUE;
			}
			else
				return FALSE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

inline BOOL CFileLog::IsOpen()
{
	return(m_hFile != INVALID_HANDLE_VALUE);
}

inline BOOL CFileLog::Flush()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
	{
		return (::FlushFileBuffers(m_hFile) != 0); 
	}
	return FALSE;
}
bool CFileLog::ShowFile()
{
	HINSTANCE hInst = ShellExecute(NULL, _T("open"), m_szLogFile, NULL, NULL, SW_SHOWNORMAL);
	return (hInst > (HINSTANCE)32);
}
