#include "StdAfx.h"
#include "FileLog.h"
#include "SocketClient.h"

SocketClient::SocketClient(void) : m_bAutoStart(false)
{
	m_szSocketKey[0] = (_TCHAR)NULL;
	m_szRemoteHost[0] = (_TCHAR)NULL;
}

SocketClient::~SocketClient(void)
{
}

bool SocketClient::GetRemoteHostName(LPTSTR lpszRemoteHostName, int& nLength)
{
	try
	{
		int nRetVal = (int)::_tcslen(m_szRemoteHost);
		if (lpszRemoteHostName == NULL)
		{
			nLength = nRetVal + 1;
			return true;
		}
		if (nRetVal == 0)
		{
			nLength = nRetVal;
			return false;
		}
		if (nRetVal + 1 <= nLength)
		{
			::_tcscpy_s(lpszRemoteHostName, nLength, m_szRemoteHost); 
			nLength = nRetVal;
			return true;
		}
		nLength = nRetVal + 1;
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::GetRemoteHostName()"));
	}
	return false;
}

bool SocketClient::SetRemoteHostName(LPTSTR lpszRemoteHostName, int nLength)
{
	try
	{
		if (IsOpen())
		{
			m_dwLastErrNumber = WSAEACCES;
			return false;
		}
		if (nLength <= MAX_HOST_LENGTH)
		{
			::_tcscpy_s(m_szRemoteHost, MAX_HOST_LENGTH, lpszRemoteHostName); 
			return true;
		}
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::SetRemoteHostName()"));
	}
	return false;
}

bool SocketClient::Initialize(HWND hWnd, int nPort, LPCTSTR lpRemoteHost)
{
	bool bRetVal = false;
	USES_CONVERSION;
	try
	{
		if (!Close(hWnd))
		{
			return bRetVal;
		}
		if (!SocketBase::Initialize(hWnd, nPort, lpRemoteHost))
		{
			return bRetVal;
		}
		if (lpRemoteHost == NULL)
		{
			if (::_tcslen(m_szRemoteHost) == 0)
			{
				::_tcscpy_s(m_szRemoteHost, A2CT(m_szHostName));
			}
		}
		else
		{
			::_tcscpy_s(m_szRemoteHost, lpRemoteHost);
		}

		LPCTSTR lpRemHost = (LPCTSTR)m_szRemoteHost;
		const char* lpszLH = (const char*)m_szHostName;
		const char* lpszRH = T2CA(lpRemHost); 
		
		LPHOSTENT lpHstLH = ::gethostbyname(lpszLH);
		if (lpHstLH == NULL)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal; 
		}
		char ipLH[MAX_LEN_IP];
		strcpy_s(ipLH, inet_ntoa(*(struct in_addr *)*lpHstLH->h_addr_list));

		/*LPHOSTENT lpHstRH = ::gethostbyname(lpszRH);
		if (lpHstRH == NULL)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal;
		}
		*char ipRH[MAX_LEN_IP];
		*strcpy_s(ipRH, inet_ntoa(*(struct in_addr *)*lpHstRH->h_addr_list));
		
		::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
		m_socket_arrd.sin_family = AF_INET;
		m_socket_arrd.sin_port = htons(m_nPort);
		::memcpy(&(m_socket_arrd.sin_addr), 
			lpHstRH->h_addr, lpHstRH->h_length);

		if(::connect(m_socket, (PSOCKADDR)&m_socket_arrd, 
			sizeof(SOCKADDR)) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
			return bRetVal;
		}*/

		char szPort[MAX_LEN_PORT];
		errno_t err = _itoa_s(GetPort(), szPort, MAX_LEN_PORT, 10);
		_ASSERTE(err == 0);
		ADDRINFO aiHints;
		LPADDRINFO aiList = NULL;
		::memset(&aiHints, 0, sizeof(ADDRINFO));
		aiHints.ai_family = AF_INET;
		aiHints.ai_socktype = SOCK_STREAM;
		aiHints.ai_protocol = IPPROTO_TCP;
		if (::getaddrinfo(lpszRH, (const char*)szPort, &aiHints, &aiList) != 0)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal;
		}
		for (LPADDRINFO pAI = aiList; pAI != NULL; pAI = pAI->ai_next)
		{
			if(::connect(m_socket, pAI->ai_addr, (int)pAI->ai_addrlen) != SOCKET_ERROR)
			{
				m_dwLastErrNumber = NO_ERROR;
				break;
			}
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
		}
		::freeaddrinfo(aiList);
		if (m_dwLastErrNumber != NO_ERROR)
		{
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal;
		}
		
		int nLenAddr = sizeof(SOCKADDR_IN);
		::memset((SOCKADDR_IN*)&m_socket_arrd, 0, nLenAddr);
		if (::getpeername(m_socket, (SOCKADDR*)&m_socket_arrd, &nLenAddr) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
			return bRetVal;
		}
		if (::WSAAsyncSelect(m_socket, hWnd, WSA_NETEVENT, FD_READ | FD_WRITE | FD_CLOSE) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
			return bRetVal; 
		}
		_TCHAR szBuffMsg[LEN_MSG_COMPLETED_REM];
		::_stprintf_s(szBuffMsg, LEN_MSG_COMPLETED_REM, FRM_MSG_COMPLETED_REM, 
			A2CT(m_szHostName), A2CT(ipLH), lpRemHost, 
			A2CT(inet_ntoa(m_socket_arrd.sin_addr)), htons(m_socket_arrd.sin_port));
		WriteMsg(szBuffMsg);
		int nLenKey = GetSocketKey((LPTSTR)m_szSocketKey, 
			(LPSOCKADDR_IN)&m_socket_arrd);
		_ASSERTE(nLenKey != 0);
		bRetVal = true;
	}
	catch (...)
	{
		if (m_socket != INVALID_SOCKET)
		{
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
		}
		ShowWin32Err(_T("SocketClient::Initialize()"));
	}
	return bRetVal;
}

bool SocketClient::Close(HWND hWnd)
{
	bool bRetVal = true;
	try
	{
		if (!IsOpen())
		{
			return bRetVal;
		}
		_TCHAR szBuffMsg[LEN_MSG_CLOSE_REM];
		::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE_REM, FRM_MSG_CLOSE_REM, m_szSocketKey);
		WriteMsg(szBuffMsg);
		//   
		//    
		::WSAAsyncSelect(m_socket, hWnd, 0, 0);
		m_szSocketKey[0] = (_TCHAR)NULL;
		bRetVal = SocketBase::Close();
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::Close()"));
	}
	return bRetVal;
}

bool SocketClient::WriteMsg(LPCTSTR lpszMsg)
{
	bool bRetVal = false;
	try
	{
		_TCHAR szBufTNow[TIME_STR_LEN];
		struct tm timenow;
		time_t long_time;
		::time(&long_time);
		errno_t err = ::localtime_s(&timenow, &long_time);
		_ASSERTE(err == 0);
		::_tcsftime(szBufTNow, TIME_STR_LEN, TIME_FORMAT, &timenow);
		::_tcscat_s(m_BuffMsg, _T("["));
		::_tcscat_s(m_BuffMsg, szBufTNow);
		::_tcscat_s(m_BuffMsg, _T("] - "));
		::_tcscat_s(m_BuffMsg, lpszMsg);
		::_tcscat_s(m_BuffMsg, _T("\r\n"));
		bRetVal = WriteLog(lpszMsg);
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::WriteMsg()"));
	}
	return bRetVal;
}

DWORD SocketClient::Send(LPCTSTR lpszMsg)
{
	DWORD dwRetVal = 0;
	if (!IsOpen())
	{
		return dwRetVal;
	}

	USES_CONVERSION;
	try
	{
		int nBytes = 0;
		if ((nBytes = ::send(m_socket, T2CA(lpszMsg), (int)::_tcslen(lpszMsg), 0)) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError();
		} else if (nBytes > 0)
		{
			_TCHAR szBuffMsg[LEN_MSG_WRITE_REM];
			::_stprintf_s(szBuffMsg, LEN_MSG_WRITE_REM, FRM_MSG_WRITE_REM, m_szSocketKey, nBytes);
			WriteMsg(szBuffMsg);
			WriteMsg(lpszMsg);
			dwRetVal = (DWORD)nBytes;
		}
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::Send()"));
	}
	return dwRetVal;
}

void SocketClient::Event_OnWSANet(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(msg);
	UNREFERENCED_PARAMETER(wParam);
	USES_CONVERSION;

	try
	{
		if((m_dwLastErrNumber = WSAGETSELECTERROR(lParam)) != NO_ERROR)
		{
			Close(hWnd);
			return;
		}
		_ASSERTE(m_socket != INVALID_SOCKET);

		switch (WSAGETSELECTEVENT(lParam))
		{
		case FD_READ:
			{
				char recvbuf[RECV_BUFF_SIZE];
				int nResult = ::recv(m_socket, recvbuf, RECV_BUFF_SIZE, 0);
				if (nResult > 0)
				{
					_TCHAR szBuffMsg[LEN_MSG_READ_REM];
					::_stprintf_s(szBuffMsg, LEN_MSG_READ_REM, FRM_MSG_READ_REM, m_szSocketKey, nResult);
					recvbuf[nResult] = (char)NULL;
					WriteMsg(szBuffMsg);
					WriteMsg(A2CT(recvbuf));
				}
				else if (nResult == 0)
				{
					Close(hWnd);
				}
				else
				{
					m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
					Close(hWnd);
				}
			}
			break;
		case FD_WRITE:
			{
				_TCHAR szSendBuf[LEN_MSG_HI];
				::_stprintf_s(szSendBuf, LEN_MSG_HI, FRM_MSG_HI, m_szSocketKey);
				int nBytes = 0;
				if ((nBytes = ::send(m_socket, T2CA(szSendBuf), (int)_tcslen(szSendBuf), 0)) == SOCKET_ERROR)
				{
					m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
				}
				else if (nBytes > 0)
				{
					_TCHAR szBuffMsg[LEN_MSG_WRITE_REM];
					::_stprintf_s(szBuffMsg, LEN_MSG_WRITE_REM, FRM_MSG_WRITE_REM, m_szSocketKey, nBytes);
					WriteMsg(szBuffMsg);
					WriteMsg(szSendBuf);
				}
			}
			break;
		case FD_CLOSE:
			{
				if ((SOCKET)wParam == m_socket)
				{
					Close(hWnd);
				}
			}
			break;
		default:
			{
				WriteLog(_T("OnWSANet - Unknown case!"));
				::MessageBox(hWnd, _T("Unknown case"), _T("Client"),
					MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
			}
			break;
		}
		if (m_dwLastErrNumber != NO_ERROR)
		{
		}
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketClient::Event_OnWSANet()"));
	}
}