#include "StdAfx.h"
#include "FileLog.h"
#include "SocketServer.h"

SocketServer::SocketServer(void) : m_bAcceptMode(false)
{
}

SocketServer::~SocketServer(void)
{
}

bool SocketServer::Initialize(HWND hWnd, int nPort, LPCTSTR lpRemoteHost)
{
	bool bRetVal = false;
	try
	{
		if (!Close(hWnd))
		{
			return bRetVal;
		}
		if (!SocketBase::Initialize(hWnd, nPort, lpRemoteHost))
		{
			return bRetVal;
		}

		LPHOSTENT thisHost = ::gethostbyname("");
		if (thisHost == NULL)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal; 
		}

		//     
		::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
		m_socket_arrd.sin_family = AF_INET;
		m_socket_arrd.sin_port = htons(m_nPort);
		::memcpy(&(m_socket_arrd.sin_addr), 
			thisHost->h_addr, thisHost->h_length);

		if(::bind(m_socket, (LPSOCKADDR)&m_socket_arrd, 
			sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal; 
		}
		if (::listen(m_socket, SOMAXCONN ) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal;
		}
		if (::WSAAsyncSelect(m_socket, hWnd, WSA_ACCEPT, FD_ACCEPT) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			return bRetVal; 
		}

		USES_CONVERSION;
		_TCHAR szBuffMsg[LEN_MSG_COMPLETED_LOC];
		::_stprintf_s(szBuffMsg, LEN_MSG_COMPLETED_LOC, FRM_MSG_COMPLETED_LOC, 
			A2CT(m_szHostName), A2CT(inet_ntoa(m_socket_arrd.sin_addr)), htons(m_socket_arrd.sin_port));
		WriteMsg(szBuffMsg);
		bRetVal = true;
	}
	catch (...)
	{
		if (m_socket != INVALID_SOCKET)
		{
			::closesocket(m_socket);
			m_socket = INVALID_SOCKET;
			::memset((SOCKADDR_IN*)&m_socket_arrd, 0, sizeof(SOCKADDR_IN));
		}
		ShowWin32Err(_T("SocketServer::Initialize()"));
	}
	return bRetVal;
}

bool SocketServer::Close(HWND hWnd)
{
	bool bRetVal = false;
	_TCHAR szBuffMsg[LEN_MSG_CLOSE];
	_TCHAR szSockKey[LEN_BUFF_KEY];
	int nLenKey = 0;
	try
	{
		if (m_list_client_sockets.size() > 0)
		{
			for(LIST_CLIENTSOCKET::iterator lstIter = m_list_client_sockets.begin();
				lstIter != m_list_client_sockets.end(); lstIter++)
			{
				if ((*lstIter).client_socket != INVALID_SOCKET)
				{
					nLenKey = GetSocketKey((LPTSTR)szSockKey, 
						(LPSOCKADDR_IN)&(*lstIter).client_socket_arrd);
					_ASSERTE(nLenKey != 0);
					::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE, FRM_MSG_CLOSE, szSockKey);
					WriteMsg(szBuffMsg);
					::WSAAsyncSelect((*lstIter).client_socket, hWnd, 0, 0);
					::closesocket((*lstIter).client_socket);
				}
			}
			m_list_client_sockets.erase(m_list_client_sockets.begin(), m_list_client_sockets.end());
		}
		if (m_socket != INVALID_SOCKET)
		{
			nLenKey = GetSocketKey((LPTSTR)szSockKey, 
				(LPSOCKADDR_IN)&m_socket_arrd);
			_ASSERTE(nLenKey != 0);
			::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE, FRM_MSG_CLOSE, szSockKey);
			WriteMsg(szBuffMsg);
			::WSAAsyncSelect(m_socket, hWnd, 0, 0);
		}
		bRetVal = SocketBase::Close();
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketServer::Close()"));
	}
	return bRetVal;
}

int SocketServer::GetClientsCount()
{
	return (int)m_list_client_sockets.size(); 
}

bool SocketServer::WriteMsg(LPCTSTR lpszMsg)
{
	bool bRetVal = false;
	_TCHAR szBufTNow[TIME_STR_LEN];
	struct tm timenow;
	try
	{
		time_t long_time;
		::time(&long_time);
		errno_t err = ::localtime_s(&timenow, &long_time);
		_ASSERTE(err == 0);
		::_tcsftime(szBufTNow, TIME_STR_LEN, TIME_FORMAT, &timenow);
		::_tcscat_s(m_BuffMsg, _T("["));
		::_tcscat_s(m_BuffMsg, szBufTNow);
		::_tcscat_s(m_BuffMsg, _T("] - "));
		::_tcscat_s(m_BuffMsg, lpszMsg);
		::_tcscat_s(m_BuffMsg, _T("\r\n"));
		bRetVal = WriteLog(lpszMsg);
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketServer::WriteMsg()"));
	}
	return bRetVal;
}

DWORD SocketServer::Send(HWND hWnd, LPCTSTR lpszMsg)
{
	DWORD dwCount = 0;
	USES_CONVERSION;
	_TCHAR szSockKey[LEN_BUFF_KEY];
	_TCHAR szBuffMsg[LEN_MSG_WRITE];
	int nLenKey = 0;
	int nBytes = 0;
	try
	{
		for(LIST_CLIENTSOCKET::iterator lstIter = m_list_client_sockets.begin();
			lstIter != m_list_client_sockets.end(); lstIter++)
		{
			if ((nBytes = ::send((*lstIter).client_socket, T2CA(lpszMsg), (int)_tcslen(lpszMsg), 0)) == SOCKET_ERROR)
			{
				m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
				ShowErrorMessage(hWnd);
			}
			else if (nBytes > 0)
			{
				nLenKey = GetSocketKey((LPTSTR)szSockKey, 
					(LPSOCKADDR_IN)&(*lstIter).client_socket_arrd);
				_ASSERTE(nLenKey != 0);
				::_stprintf_s(szBuffMsg, LEN_MSG_WRITE, FRM_MSG_WRITE, szSockKey, nBytes);
				WriteMsg(szBuffMsg);
				WriteMsg(lpszMsg);
				dwCount++;
			}
		}
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketServer::Send()"));
	}
	return dwCount;
}

void SocketServer::Event_OnWSAAccept(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(msg);
	UNREFERENCED_PARAMETER(wParam);

	try
	{
		if((m_dwLastErrNumber = WSAGETSELECTERROR(lParam)) != NO_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError();
			ShowErrorMessage(hWnd);
			Close(hWnd);
			return;
		}

		_TCHAR szSockKey[LEN_BUFF_KEY];
		int nLenKey = 0;

		//    
		//int acc_sin_len = sizeof(SOCKADDR_IN);
		int acc_sin_len = sizeof(SOCKADDR);

		//   
		LIST_CLIENTSOCKET::iterator lstIter 
			= m_list_client_sockets.insert(m_list_client_sockets.end(), CLIENTSOCKET());
		SOCKET* lpClient_socket = &(*lstIter).client_socket;
		LPSOCKADDR_IN lpClient_socket_arrd = &(*lstIter).client_socket_arrd;
		if ((*lpClient_socket = ::accept(m_socket, (LPSOCKADDR)lpClient_socket_arrd,
			(int FAR *)&acc_sin_len)) == INVALID_SOCKET)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			_TCHAR szBuffMsg[LEN_MSG_CLOSE];
			nLenKey = GetSocketKey((LPTSTR)szSockKey, 
				(LPSOCKADDR_IN)&m_socket_arrd);
			_ASSERTE(nLenKey != 0);
			::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE, FRM_MSG_CLOSE, szSockKey);
			WriteMsg(szBuffMsg);
			m_list_client_sockets.erase(lstIter);  
			ShowErrorMessage(hWnd);
			return;
		}

		//        
		// ,       WSA_NETEVENT.
		//       
		if (::WSAAsyncSelect(*lpClient_socket, hWnd, WSA_NETEVENT, 
			FD_READ | FD_WRITE | FD_CLOSE) == SOCKET_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			_TCHAR szBuffMsg[LEN_MSG_CLOSE];
			nLenKey = GetSocketKey((LPTSTR)szSockKey, 
				(LPSOCKADDR_IN)lpClient_socket_arrd);
			_ASSERTE(nLenKey != 0);
			::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE, FRM_MSG_CLOSE, szSockKey);
			WriteMsg(szBuffMsg);
			::closesocket(*lpClient_socket);
			*lpClient_socket = INVALID_SOCKET;
			m_list_client_sockets.erase(lstIter);  
			ShowErrorMessage(hWnd);
		}

		USES_CONVERSION;
		_TCHAR szBuffMsg[LEN_MSG_ACCEPT];
		nLenKey = GetSocketKey((LPTSTR)szSockKey, 
			(LPSOCKADDR_IN)lpClient_socket_arrd);
		_ASSERTE(nLenKey != 0);
		::_stprintf_s(szBuffMsg, LEN_MSG_ACCEPT, FRM_MSG_ACCEPT, szSockKey);
		WriteMsg(szBuffMsg);
		m_bAcceptMode = true;
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketServer::Event_OnWSAAccept()"));
	}
}

void SocketServer::Event_OnWSANet(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(msg);
	
	try
	{
		SOCKET& lClient_socket = (SOCKET&)wParam;
		_ASSERTE(lClient_socket != INVALID_SOCKET);
		LIST_CLIENTSOCKET::iterator lstIter = NULL;
		bool bFound = false;
		if (m_list_client_sockets.size() > 0)
		{
			for(lstIter = m_list_client_sockets.begin();
				lstIter != m_list_client_sockets.end(); lstIter++)
			{
				if ((*lstIter).client_socket == lClient_socket)
				{
					bFound = true;
					break;
				}
			}
		}
		_ASSERTE(bFound == true);
		SOCKADDR_IN& lClient_socket_arrd = (SOCKADDR_IN&)(*lstIter).client_socket_arrd;

		USES_CONVERSION;

		bool bReadEvent = false;
		bool bIsClosed = false;

		if((m_dwLastErrNumber = WSAGETSELECTERROR(lParam)) != NO_ERROR)
		{
			m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
			bIsClosed = true;
		}
		else
		{
			switch (WSAGETSELECTEVENT(lParam))
			{
			case FD_READ:
				{
					bReadEvent = true;
					char recvbuf[RECV_BUFF_SIZE];
					int nResult = ::recv(lClient_socket, recvbuf, RECV_BUFF_SIZE, 0);
					if (nResult > 0)
					{
						_TCHAR szBuffMsg[LEN_MSG_READ];
						_TCHAR szSockKey[LEN_BUFF_KEY];
						int nLenKey = GetSocketKey((LPTSTR)szSockKey, 
							(LPSOCKADDR_IN)&lClient_socket_arrd);
						_ASSERTE(nLenKey != 0);
						::_stprintf_s(szBuffMsg, LEN_MSG_READ, FRM_MSG_READ, szSockKey, nResult);
						WriteMsg(szBuffMsg);
						recvbuf[nResult] = (char)NULL;
						WriteMsg(A2CT(recvbuf));
					}
					else if (nResult == 0)
					{
						bIsClosed = true;
					}
					else
					{
						m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
						if (m_dwLastErrNumber == WSAEWOULDBLOCK || !m_bAcceptMode)
						{
							//   ,    
							m_dwLastErrNumber = NO_ERROR;
						}
						else
						{
							bIsClosed = true;
						}
					}
				}
				break;
			case FD_WRITE:
				{
					_TCHAR szSockKey[LEN_BUFF_KEY];
					_TCHAR szSendBuf[LEN_MSG_HI];
					int nLenKey = GetSocketKey((LPTSTR)szSockKey, 
						(LPSOCKADDR_IN)&lClient_socket_arrd);
					_ASSERTE(nLenKey != 0);
					::_stprintf_s(szSendBuf, LEN_MSG_HI, FRM_MSG_HI, szSockKey);
					int nBytes = 0;
					if ((nBytes = ::send(lClient_socket, T2CA(szSendBuf), (int)_tcslen(szSendBuf), 0)) == SOCKET_ERROR)
					{
						m_dwLastErrNumber = (DWORD)::WSAGetLastError(); 
					}
					else if (nBytes > 0)
					{
						_TCHAR szBuffMsg[LEN_MSG_WRITE];
						::_stprintf_s(szBuffMsg, LEN_MSG_WRITE, FRM_MSG_WRITE, szSockKey, nBytes);
						WriteMsg(szBuffMsg);
						WriteMsg(szSendBuf);
					}
				}
				break;
			case FD_CLOSE:
				{
					bIsClosed = true;
				}
				break;
			default:
				{
					WriteLog(_T("OnWSANet - Unknown case!"));
					::MessageBox(hWnd, _T("Unknown case"), _T("Server"),
						MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
				}
				break;
			}
		}
		if (bReadEvent || bIsClosed)
		{
			if (m_bAcceptMode)
			{
				m_bAcceptMode = false;
			}
		}
		if (bIsClosed)
		{
			_TCHAR szSockKey[LEN_BUFF_KEY];
			_TCHAR szBuffMsg[LEN_MSG_CLOSE];
			int nLenKey = GetSocketKey((LPTSTR)szSockKey, 
				(LPSOCKADDR_IN)&lClient_socket_arrd);
			_ASSERTE(nLenKey != 0);
			::_stprintf_s(szBuffMsg, LEN_MSG_CLOSE, FRM_MSG_CLOSE, szSockKey);
			WriteMsg(szBuffMsg);
			::closesocket(lClient_socket);
			for(LIST_CLIENTSOCKET::iterator lstIter = m_list_client_sockets.begin();
				lstIter != m_list_client_sockets.end(); lstIter++)
			{
				if ((*lstIter).client_socket == lClient_socket)
				{
					m_list_client_sockets.erase(lstIter); 
					break;
				}
			}
		}
		if (m_dwLastErrNumber != NO_ERROR)
		{
		}
	}
	catch (...)
	{
		ShowWin32Err(_T("SocketServer::Event_OnWSANet()"));
	}
}